package com.jiuqi.office.excel;

import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import com.jiuqi.grid.CellField;
import com.jiuqi.grid.GridFieldList;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.Color;
import com.jiuqi.grid.CurrencyCellProperty;
import java.util.Date;
import org.apache.poi.ss.usermodel.Row;
import java.text.NumberFormat;
import com.jiuqi.grid.NumberCellProperty;
import com.jiuqi.grid.GridCell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.RichTextString;
import org.apache.poi.hssf.util.CellRangeAddress;
import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.apache.poi.ss.usermodel.Font;
import com.jiuqi.grid.GridData;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

public final class WorksheetWriter
{
    private Workbook workBook;
    private Sheet sheet;
    private GridData gridData;
    private String title;
    private boolean autoAdjust;
    private boolean addTitle;
    private Font titleFont;
    private HashMap cellStyleHashMap;
    private int transparentColor;
    private boolean doNotCheckNumberIsNull;
    private Map HSSFFontMap;
    private List sheets;
    private static int MAX_COLUMN;
    private String preSheetName;
    
    static {
        WorksheetWriter.MAX_COLUMN = 255;
    }
    
    public WorksheetWriter(final HSSFWorkbook workBook, final HSSFSheet sheet, final GridData gridData, final HashMap cellStyleHashMap) {
        this.sheets = new ArrayList();
        this.workBook = (Workbook)workBook;
        this.sheet = (Sheet)sheet;
        this.sheets.add(sheet);
        this.gridData = gridData;
        this.cellStyleHashMap = cellStyleHashMap;
        this.HSSFFontMap = new HashMap();
        this.transparentColor = -16777201;
    }
    
    public WorksheetWriter(final Workbook workBook, final Sheet sheet, final GridData gridData, final HashMap cellStyleHashMap) {
        this.sheets = new ArrayList();
        this.workBook = workBook;
        this.sheet = sheet;
        this.sheets.add(sheet);
        this.gridData = gridData;
        this.cellStyleHashMap = cellStyleHashMap;
        this.HSSFFontMap = new HashMap();
        this.transparentColor = -16777201;
    }
    
    public String getPreSheetName() {
        return this.preSheetName;
    }
    
    public void setPreSheetName(final String preSheetName) {
        this.preSheetName = preSheetName;
    }
    
    public void setTitle(final String title) {
        this.title = title;
    }
    
    private void setDefaultTitleFont() {
        (this.titleFont = this.workBook.createFont()).setFontName("宋体");
        this.titleFont.setBoldweight((short)700);
        this.titleFont.setFontHeightInPoints((short)12);
    }
    
    public void setAutoAdjust(final boolean autoAdjust) {
        this.autoAdjust = autoAdjust;
    }
    
    public void setAddTitle(final boolean addTitle) {
        this.addTitle = addTitle;
    }
    
    public void setTransparentColor(final int transparentColor) {
        this.transparentColor = transparentColor;
    }
    
    public boolean isDoNotCheckNumberIsNull() {
        return this.doNotCheckNumberIsNull;
    }
    
    public void setDoNotCheckNumberIsNull(final boolean doNotCheckNumberIsNull) {
        this.doNotCheckNumberIsNull = doNotCheckNumberIsNull;
    }
    
    public void writeWorkSheet() {
        this.addSheets();
        this.addTitle();
        this.writeContent();
        this.adjustHW();
        this.mergeCells();
        this.hideRowCol();
    }
    
    private Sheet getSheet(final int index) {
        return this.sheets.get(index);
    }
    
    private int getSheetIndex(final int col) {
        return col / WorksheetWriter.MAX_COLUMN;
    }
    
    private int getCol(final int col) {
        return col % WorksheetWriter.MAX_COLUMN;
    }
    
    private void addSheets() {
        if (this.gridData.getColCount() > WorksheetWriter.MAX_COLUMN) {
            for (int extraSheetsNum = (this.gridData.getColCount() + WorksheetWriter.MAX_COLUMN - 1) / WorksheetWriter.MAX_COLUMN - 1, i = 0; i < extraSheetsNum; ++i) {
                this.sheets.add(this.workBook.createSheet(String.valueOf(this.preSheetName) + (i + 1)));
            }
        }
    }
    
    private void addTitle() {
        if (this.addTitle) {
            this.setDefaultTitleFont();
            final RichTextString richText = this.getRichTextString(this.title, this.workBook);
            for (int i = 0; i < this.sheets.size(); ++i) {
                final Cell cell = this.getSheet(i).createRow(0).createCell(0);
                if (richText != null) {
                    cell.setCellValue(richText);
                }
                final CellStyle titleCellStyle = this.workBook.createCellStyle();
                titleCellStyle.setAlignment((short)2);
                titleCellStyle.setFont(this.titleFont);
                cell.setCellStyle(titleCellStyle);
                this.getSheet(i).addMergedRegion((org.apache.poi.ss.util.CellRangeAddress)new CellRangeAddress(0, 0, 0, Math.min(this.gridData.getColCount() - 2, WorksheetWriter.MAX_COLUMN)));
            }
        }
    }
    
    private void writeContent() {
        if (this.gridData == null) {
            throw new NullPointerException("表格数据不能为空");
        }
        final int rowCount = this.gridData.getRowCount();
        final int colCount = this.gridData.getColCount();
        for (int row = 1; row < rowCount; ++row) {
            for (int col = 1; col < colCount; ++col) {
                final GridCell gridCell = this.gridData.getCell(col, row);
                this.writeCell(gridCell);
            }
        }
    }
    
    private void writeCell(final GridCell gridCell) {
        int rowNum = gridCell.getRowNum() - 1;
        final int colNum = gridCell.getColNum() - 1;
        if (this.addTitle) {
            ++rowNum;
        }
        final Row row = (this.getSheet(this.getSheetIndex(colNum)).getRow(rowNum) == null) ? this.getSheet(this.getSheetIndex(colNum)).createRow(rowNum) : this.getSheet(this.getSheetIndex(colNum)).getRow(rowNum);
        final Cell cell = row.createCell(this.getCol(colNum));
        try {
            final CellStyle cellStyle = this.buildCellStyle(gridCell);
            cell.setCellStyle(cellStyle);
            switch (gridCell.getDataType()) {
                case 0:
                case 1:
                case 4: {
                    final String strValue = gridCell.getCellData();
                    final RichTextString richTextString = this.getRichTextString(strValue, this.workBook);
                    if (richTextString != null) {
                        cell.setCellValue(richTextString);
                        break;
                    }
                    break;
                }
                case 2: {
                    final String cellData = gridCell.getCellData();
                    if (cellData == null || cellData.equals("")) {
                        break;
                    }
                    final double value = gridCell.getFloat();
                    cell.setCellValue(value);
                    if (!gridCell.getShowText().endsWith("‰")) {
                        break;
                    }
                    final NumberCellProperty ncp = new NumberCellProperty(gridCell);
                    final String decimalStr = String.valueOf(this.buildDecimal(ncp, -3)) + "‰";
                    final NumberFormat numf = NumberFormat.getInstance();
                    numf.setGroupingUsed(true);
                    final RichTextString richTextString2 = this.getRichTextString(numf.format(value * 1000.0), this.workBook);
                    if (richTextString2 == null) {
                        cell.setCellValue(richTextString2 + decimalStr);
                        break;
                    }
                    cell.setCellValue(decimalStr);
                    break;
                }
                case 5: {
                    final Date date = gridCell.getDateTime();
                    if (date != null) {
                        cell.setCellValue(date);
                        break;
                    }
                    break;
                }
                case 3: {
                    final String cellData = gridCell.getCellData();
                    if (cellData == null || cellData.equals("")) {
                        break;
                    }
                    if (!"|".equals(cellData)) {
                        final double value = Double.parseDouble(cellData);
                        cell.setCellValue(value);
                        break;
                    }
                    final String value2 = gridCell.getShowText();
                    final RichTextString richTextString3 = this.getRichTextString(value2, this.workBook);
                    if (richTextString3 != null) {
                        cell.setCellValue(richTextString3);
                        break;
                    }
                    break;
                }
                case 9: {
                    final String linkInfoStr = gridCell.getCellData();
                    if (linkInfoStr == null || "".equals(linkInfoStr)) {
                        break;
                    }
                    final String[] info = GridCell.parserLinkInformation(linkInfoStr);
                    final RichTextString hssfStr = this.getRichTextString(info[0], this.workBook);
                    if (hssfStr != null) {
                        cell.setCellValue(hssfStr);
                        break;
                    }
                    break;
                }
            }
        }
        catch (final Exception e) {
            e.printStackTrace();
        }
    }
    
    private String makeCurFotmat(final CurrencyCellProperty curCellProperty) {
        String currencyFormat = "#,##0";
        String colorStr = "";
        boolean isBracketNegative = false;
        currencyFormat = String.valueOf(currencyFormat) + this.buildDecimal((NumberCellProperty)curCellProperty, 0);
        final int idx = curCellProperty.getUnitIndex();
        if (curCellProperty.getWarningNegative()) {
            colorStr = "[Red]";
        }
        isBracketNegative = curCellProperty.getBracketNegative();
        switch (curCellProperty.getUnitShowType()) {
            case 0: {
                final String signStr = CurrencyCellProperty.CURRENCY_UNIT_SIGNS[idx];
                if (signStr.length() == 1) {
                    currencyFormat = (isBracketNegative ? (String.valueOf(signStr) + currencyFormat + ";" + colorStr + signStr + "(" + currencyFormat + ")") : (String.valueOf(signStr) + currencyFormat + ";" + colorStr + signStr + "-" + currencyFormat));
                    break;
                }
                if (signStr.length() > 1) {
                    currencyFormat = (isBracketNegative ? ("[$" + signStr + "]" + currencyFormat + ";" + colorStr + "[$" + signStr + "](" + currencyFormat + ")") : ("[$" + signStr + "]" + currencyFormat + ";" + colorStr + "[$" + signStr + "]-" + currencyFormat));
                    break;
                }
                break;
            }
            case 1: {
                final String nameStr = CurrencyCellProperty.CURRENCY_UNIT_NAMES[idx];
                currencyFormat = (isBracketNegative ? ("[$" + nameStr + "] " + currencyFormat + ";" + colorStr + "[$" + nameStr + "] (" + currencyFormat + ")") : ("[$" + nameStr + "] " + currencyFormat + ";" + colorStr + "[$" + nameStr + "] -" + currencyFormat));
                break;
            }
            case 2: {
                final String nameStr = CurrencyCellProperty.CURRENCY_UNIT_NAMES[idx];
                currencyFormat = (isBracketNegative ? (String.valueOf(currencyFormat) + " [$" + nameStr + "];" + colorStr + "(" + currencyFormat + ") [$" + nameStr + "]") : (String.valueOf(currencyFormat) + " [$" + nameStr + "];" + colorStr + "-" + currencyFormat + " [$" + nameStr + "]"));
                break;
            }
            case 3: {
                final String titleStr = CurrencyCellProperty.CURRENCY_UNIT_TITLES[idx];
                currencyFormat = (isBracketNegative ? ("[$" + titleStr + "] " + currencyFormat + ";" + colorStr + "[$" + titleStr + "] (" + currencyFormat + ")") : ("[$" + titleStr + "] " + currencyFormat + ";" + colorStr + "[$" + titleStr + "] -" + currencyFormat));
                break;
            }
        }
        return currencyFormat;
    }
    
    private String buildDecimal(final NumberCellProperty numberCellProperty, final int offset) {
        final int decimalCount = numberCellProperty.getDecimal() + offset;
        if (decimalCount > 0) {
            String decimal = ".";
            for (int i = 0; i < decimalCount; ++i) {
                decimal = String.valueOf(decimal) + "0";
            }
            return decimal;
        }
        return "";
    }
    
    private Font getCellFont(final GridCell gridCell) {
        final String fontKey = this.getCellFontKey(gridCell);
        Font font = this.HSSFFontMap.get(fontKey);
        if (font == null) {
            font = this.workBook.createFont();
            this.HSSFFontMap.put(fontKey, font);
            if (gridCell.getFontBold()) {
                font.setBoldweight((short)700);
            }
            final Color fontColor = (Color)HSSFHelper.getApproximateTextColor(gridCell.getFontColor());
            if (fontColor != null) {}
            font.setFontHeight((short)gridCell.getFontHeight());
            font.setItalic(gridCell.getFontItalic());
            font.setFontName(gridCell.getFontName());
            font.setFontHeightInPoints((short)gridCell.getFontSize());
            font.setStrikeout(gridCell.getFontStrikeOut());
            if (gridCell.getFontUnderLine()) {
                font.setUnderline((byte)1);
            }
        }
        return font;
    }
    
    private String getCellFontKey(final GridCell cell) {
        final StringBuffer key = new StringBuffer();
        key.append(cell.getFontName());
        key.append(cell.getFontSize());
        key.append(cell.getFontColor());
        key.append(cell.getFontHeight());
        key.append(cell.getFontBold());
        key.append(cell.getFontItalic());
        key.append(cell.getFontStrikeOut());
        key.append(cell.getFontUnderLine());
        return key.toString();
    }
    
    private CellStyle buildCellStyle(final GridCell gridCell) {
        CellStyle cellStyle = this.cellStyleHashMap.get(gridCell);
        if (cellStyle != null) {
            return cellStyle;
        }
        final GridCell tempGridCell = this.gridData.getCellEx(gridCell.getColNum(), gridCell.getRowNum());
        cellStyle = this.workBook.createCellStyle();
        short backColorIdx = 9;
        if (tempGridCell.getSilverHead()) {
            backColorIdx = 22;
        }
        else if (tempGridCell.getBackStyle() != 0) {
            final HSSFColor backColor = HSSFHelper.getApproximateColor(tempGridCell.getBackColor());
            if (backColor != null) {
                backColorIdx = backColor.getIndex();
            }
        }
        cellStyle.setFillForegroundColor(backColorIdx);
        if (backColorIdx != 58) {
            cellStyle.setFillPattern((short)1);
        }
        final short leftBorder = this.transBorderStyle(tempGridCell.getLEdgeStyle());
        final short topBorder = this.transBorderStyle(tempGridCell.getTEdgeStyle());
        final short rightBorder = this.transBorderStyle(tempGridCell.getREdgeStyle());
        final short bottomBorder = this.transBorderStyle(tempGridCell.getBEdgeStyle());
        final HSSFColor leftBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getLEdgeColor());
        final HSSFColor topBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getTEdgeColor());
        final HSSFColor righttBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getREdgeColor());
        final HSSFColor bottomBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getBEdgeColor());
        cellStyle.setBorderLeft(leftBorder);
        cellStyle.setBorderTop(topBorder);
        cellStyle.setBorderRight(rightBorder);
        cellStyle.setBorderBottom(bottomBorder);
        if (leftBorderColor != null) {
            cellStyle.setLeftBorderColor(leftBorderColor.getIndex());
        }
        if (topBorderColor != null) {
            cellStyle.setTopBorderColor(topBorderColor.getIndex());
        }
        if (righttBorderColor != null) {
            cellStyle.setRightBorderColor(righttBorderColor.getIndex());
        }
        if (bottomBorderColor != null) {
            cellStyle.setBottomBorderColor(bottomBorderColor.getIndex());
        }
        final short horzAlign = this.transCellAlign(tempGridCell.getHorzAlign(), true);
        final short vertAlign = this.transCellAlign(tempGridCell.getVertAlign(), false);
        cellStyle.setAlignment(horzAlign);
        cellStyle.setVerticalAlignment(vertAlign);
        cellStyle.setWrapText(tempGridCell.getWrapLine());
        cellStyle.setIndention((short)tempGridCell.getIndent());
        final Font font = this.getCellFont(tempGridCell);
        cellStyle.setFont(font);
        if (tempGridCell.getDataType() == 1) {
            final short formatIndex = this.workBook.createDataFormat().getFormat("@");
            cellStyle.setDataFormat(formatIndex);
        }
        if (tempGridCell.getDataType() == 2 && (this.doNotCheckNumberIsNull || tempGridCell.getShowText() != null)) {
            if (tempGridCell.getShowText() == null) {
                final NumberCellProperty ncp = new NumberCellProperty(tempGridCell);
                String formatStr = "0";
                if (ncp.getThoundsMark()) {
                    formatStr = "#,##0" + this.buildDecimal(ncp, 0);
                }
                else {
                    formatStr = String.valueOf(formatStr) + this.buildDecimal(ncp, 0);
                }
                final short formatIndex2 = this.workBook.createDataFormat().getFormat(String.valueOf(formatStr) + " ");
                cellStyle.setDataFormat(formatIndex2);
            }
            else if (tempGridCell.getShowText().endsWith("%")) {
                final NumberCellProperty ncp = new NumberCellProperty(tempGridCell);
                final String formatStr = "0" + this.buildDecimal(ncp, -2) + "%";
                final short formatIndex2 = this.workBook.createDataFormat().getFormat(formatStr);
                cellStyle.setDataFormat(formatIndex2);
            }
            else if (tempGridCell.getShowText().endsWith("‰")) {
                final NumberCellProperty ncp = new NumberCellProperty(gridCell);
                String numStr = "0";
                String colorStr = "";
                if (ncp.getThoundsMark()) {
                    numStr = "#,##0" + this.buildDecimal(ncp, 0);
                }
                else {
                    numStr = String.valueOf(numStr) + this.buildDecimal(ncp, 0);
                }
                if (ncp.getWarningNegative()) {
                    colorStr = "[Red]";
                }
                String formatStr2 = "";
                if (ncp.getBracketNegative()) {
                    formatStr2 = String.valueOf(numStr) + ";" + colorStr + "(" + numStr + ")";
                }
                else {
                    formatStr2 = String.valueOf(numStr) + ";" + colorStr + "-" + numStr;
                }
                final short formatIndex3 = this.workBook.createDataFormat().getFormat(formatStr2);
                cellStyle.setDataFormat(formatIndex3);
            }
            else {
                final NumberCellProperty ncp = new NumberCellProperty(tempGridCell);
                String formatStr = "0";
                if (ncp.getThoundsMark()) {
                    formatStr = "#,##0" + this.buildDecimal(ncp, 0);
                }
                else {
                    formatStr = String.valueOf(formatStr) + this.buildDecimal(ncp, 0);
                }
                final short formatIndex2 = this.workBook.createDataFormat().getFormat(String.valueOf(formatStr) + " ");
                cellStyle.setDataFormat(formatIndex2);
            }
        }
        if (tempGridCell.getDataType() == 3) {
            final CurrencyCellProperty curCellProperty = new CurrencyCellProperty(tempGridCell);
            final String currencyFormat = this.makeCurFotmat(curCellProperty);
            final short formatIndex2 = this.workBook.createDataFormat().getFormat(currencyFormat);
            cellStyle.setDataFormat(formatIndex2);
        }
        if (gridCell.getDataType() == 5) {
            final short formatIndex = this.workBook.createDataFormat().getFormat("yyyy-m-d");
            cellStyle.setDataFormat(formatIndex);
        }
        gridCell.getDataType();
        this.cellStyleHashMap.put(tempGridCell, cellStyle);
        return cellStyle;
    }
    
    private short transCellAlign(final int align, final boolean isHorizontal) {
        if (isHorizontal) {
            switch (align) {
                case 0: {
                    return 0;
                }
                case 1: {
                    return 1;
                }
                case 2: {
                    return 3;
                }
                case 3: {
                    return 2;
                }
                case 4:
                case 5: {
                    return 5;
                }
                case 6: {
                    return 3;
                }
                default: {
                    return 0;
                }
            }
        }
        else {
            switch (align) {
                case 0: {
                    return 1;
                }
                case 1: {
                    return 0;
                }
                case 2: {
                    return 2;
                }
                case 3:
                case 4:
                case 5: {
                    return 3;
                }
                case 6: {
                    return 2;
                }
                default: {
                    return 1;
                }
            }
        }
    }
    
    private short transBorderStyle(final int idx) {
        switch (idx) {
            case 0: {
                return 1;
            }
            case 1: {
                return 0;
            }
            case 2: {
                return 1;
            }
            case 3: {
                return 3;
            }
            case 4: {
                return 7;
            }
            case 5: {
                return 9;
            }
            case 6: {
                return 11;
            }
            case 7:
            case 8:
            case 9: {
                return 5;
            }
            case 10:
            case 11:
            case 12: {
                return 6;
            }
            case 13:
            case 14:
            case 15: {
                return 5;
            }
            default: {
                return 1;
            }
        }
    }
    
    private void mergeCells() {
        final GridFieldList fieldList = this.gridData.merges();
        for (int i = 0; i < fieldList.count(); ++i) {
            final CellField field = fieldList.get(i);
            int rowFrom = field.top - 1;
            int rowTo = field.bottom - 1;
            final int colFrom = field.left - 1;
            final int colTo = field.right - 1;
            if (this.addTitle) {
                ++rowFrom;
                ++rowTo;
            }
            final int sheetFrom = this.getSheetIndex(colFrom);
            for (int sheetTo = this.getSheetIndex(colTo), j = sheetFrom; j <= sheetTo; ++j) {
                final int curColFrom = (j == sheetFrom) ? (colFrom - j * WorksheetWriter.MAX_COLUMN) : 0;
                final int curColTo = (j == sheetTo) ? (colTo - j * WorksheetWriter.MAX_COLUMN) : WorksheetWriter.MAX_COLUMN;
                this.getSheet(j).addMergedRegion((org.apache.poi.ss.util.CellRangeAddress)new CellRangeAddress(rowFrom, rowTo, curColFrom, curColTo));
            }
        }
    }
    
    private void adjustHW() {
        final int colCount = this.gridData.getColCount();
        final int rowCount = this.gridData.getRowCount();
        if (this.autoAdjust) {
            for (int col = 0; col < colCount; ++col) {
                this.getSheet(this.getSheetIndex(col)).autoSizeColumn((int)(short)this.getCol(col));
            }
        }
        else {
            for (int col = 1; col < colCount; ++col) {
                final int colWidth = this.gridData.getColWidths(col);
                this.getSheet(this.getSheetIndex(col)).setColumnWidth(this.getCol(col) - 1, HSSFHelper.pixelToWidth(colWidth));
            }
            for (int row = 1; row < rowCount; ++row) {
                if (this.gridData.getRowVisible(row)) {
                    for (int i = 0; i < this.sheets.size(); ++i) {
                        final int rowHeight = this.gridData.getRowHeights(row);
                        final Row hssfRow = this.addTitle ? this.getSheet(i).getRow(row) : this.getSheet(i).getRow(row - 1);
                        if (hssfRow != null) {
                            hssfRow.setHeight(HSSFHelper.pixelToHeight(rowHeight));
                        }
                    }
                }
            }
        }
    }
    
    private void hideRowCol() {
        for (int col = 1; col < this.gridData.getColCount(); ++col) {
            if (!this.gridData.getColVisible(col)) {
                this.getSheet(this.getSheetIndex(col)).setColumnHidden(this.getCol(col) - 1, true);
            }
        }
        for (int row = 1; row < this.gridData.getRowCount(); ++row) {
            if (!this.gridData.getRowVisible(row)) {
                for (int i = 0; i < this.sheets.size(); ++i) {
                    final Row hssfRow = this.addTitle ? this.getSheet(i).getRow(row) : this.getSheet(i).getRow(row - 1);
                    hssfRow.setZeroHeight(true);
                }
            }
        }
    }
    
    public void setHSSFFontMap(final Map hSSFFontMap) {
        this.HSSFFontMap = hSSFFontMap;
    }
    
    public Map getHSSFFontMap() {
        return this.HSSFFontMap;
    }
    
    private RichTextString getRichTextString(final String title, final Workbook wb) {
        RichTextString rt = null;
        if (wb instanceof HSSFWorkbook) {
            rt = (RichTextString)new HSSFRichTextString(title);
        }
        else if (wb instanceof XSSFWorkbook) {
            rt = (RichTextString)new XSSFRichTextString(title);
        }
        else if (wb instanceof SXSSFWorkbook) {
            rt = (RichTextString)new XSSFRichTextString(title);
        }
        return rt;
    }
}
